package com.simpleinventory.app.db;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import java.util.LinkedList;
import java.util.List;
//http://hmkcode.com/android-simple-sqlite-database-tutorial/

public class DataBaseHelper extends SQLiteOpenHelper {

	private static final int DATABASE_VERSION = 1;
	private static final String DATABASE_NAME = "StockDB";
	private static final String TABLE_STOCK = "stock";
	private static final String KEY_ID = "id";
	private static final String KEY_SKU= "sku";
	private static final String KEY_EXPIRY_DATE= "exp_date";
	private static final String KEY_DATE_RECEIVED = "date_received";
	private static final String KEY_SERIAL_NUMBER= "serial_no";
	private static final String KEY_IS_DISPATCHED= "is_dispatched";
	private static final String[] COLUMNS = {KEY_ID,KEY_SKU,KEY_DATE_RECEIVED,KEY_SERIAL_NUMBER,KEY_IS_DISPATCHED};

	public DataBaseHelper(Context context)
	{
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db)
	{
		String CREATE_STOCK_TABLE = "CREATE TABLE stock ( " +
				"id INTEGER PRIMARY KEY AUTOINCREMENT, " +
				"sku TEXT, "+
				"exp_date TEXT, " +
				"date_received TEXT, " +
				"serial_no INTEGER, " +
				"is_dispatched INTEGER)";

		db.execSQL(CREATE_STOCK_TABLE);
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
	{
		db.execSQL("DROP TABLE IF EXISTS stock");
		this.onCreate(db);
	}

	public  void cleanDB()
	{
		SQLiteDatabase db = this.getWritableDatabase();
		db.execSQL("DELETE FROM stock");
		db.close();
	}

	public void addStock(Stock stock)
	{
		//1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		//2. create ContentValues to add key "column"/value
		ContentValues values = new ContentValues();
		values.put(KEY_SKU, stock.getSKU());
		values.put(KEY_EXPIRY_DATE, stock.getexpiryDate());
		values.put(KEY_DATE_RECEIVED, stock.getreceivedDate());
		values.put(KEY_SERIAL_NUMBER, stock.getserialNumber());
		values.put(KEY_IS_DISPATCHED, stock.getDispatch());
		// 3. insert
		db.insert(TABLE_STOCK, // table
				null, //nullColumnHack
				values); // key/value -> keys = column names/ values = column values
		// 4. close
		db.close();
	}


	public Stock getStockWhereHighestSnoWithinDate(String date)
	{
		SQLiteDatabase db = this.getReadableDatabase();
		String query = "SELECT  id, sku, exp_date, date_received, serial_no, is_dispatched  FROM " + TABLE_STOCK+" WHERE date_received='"+date+"' AND serial_no=(SELECT max(serial_no) FROM stock WHERE date_received='"+date+"')";

		Cursor cursor = db.rawQuery(query,null);
		// 3. if we got results get the first one
		if (cursor != null)
			cursor.moveToFirst();

		Stock stock = new Stock();

		if(cursor.getCount()>0)
		{
			stock = new Stock();
			stock.setID(Integer.parseInt(cursor.getString(0)));
			stock.setSKU(cursor.getString(1));
			stock.setexpiryDate(cursor.getString(2));
			stock.setreceivedDate(cursor.getString(3));
			stock.setserialNumber(cursor.getInt(4));
			stock.setDispatch(cursor.getInt(5));
		}
		db.close();
		return stock;
	}

	public List<Stock>  getStockWhere(String quer)
	{
		List<Stock> stocks = new LinkedList<Stock>();
		String query = quer;
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);
		Stock stock = null;

		if (cursor.moveToFirst())
		{
			do
			{
				stock = new Stock();
				stock.setID(Integer.parseInt(cursor.getString(0)));
				stock.setSKU(cursor.getString(1));
				stock.setexpiryDate(cursor.getString(2));
				stock.setreceivedDate(cursor.getString(3));
				stock.setserialNumber(cursor.getInt(4));
				stock.setDispatch(cursor.getInt(5));

				stocks.add(stock);
			} while (cursor.moveToNext());
		}
		// return books*/
		db.close();
		return stocks;
	}


	public int  updateDispatch(String is_dispatched, String where)
	{
		SQLiteDatabase db = this.getWritableDatabase();
		ContentValues values = new ContentValues();
		values.put("is_dispatched", is_dispatched);
		int i =db.update(TABLE_STOCK, values, where, null);
		db.close();
		return i;
	}

	public List<Stock> getAllStock()
	{
		List<Stock> stocks = new LinkedList<Stock>();
		// 1. build the query
		String query = "SELECT  * FROM " + TABLE_STOCK;
		// 2. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);
		// 3. go over each row, build book and add it to list
		Stock stock = null;

		if (cursor.moveToFirst())
		{
			do
			{
				stock = new Stock();
				stock.setID(Integer.parseInt(cursor.getString(0)));
				stock.setSKU(cursor.getString(1));
				stock.setexpiryDate(cursor.getString(2));
				stock.setreceivedDate(cursor.getString(3));
				stock.setserialNumber(cursor.getInt(4));
				stock.setDispatch(cursor.getInt(5));

				stocks.add(stock);
			} while (cursor.moveToNext());
		}

		// return books*/
		db.close();
		return stocks;
	}

}
